# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import, print_function

# import collections
import mmap
import os
import sys
import tempfile

this_file = os.path.abspath(__file__)
this_dir = os.path.split(this_file)[0]
root_dir = os.path.split(this_dir)[0]
pywayland_dir = os.path.join(root_dir, "pywayland")
if os.path.exists(pywayland_dir):
    sys.path.append(root_dir)

from pywayland.client import Display  # noqa: E402
from pywayland.protocol.wayland import (  # noqa: E402
    WlCompositor,
    WlSeat,
    WlShell,
    WlShm,
)


def create_shm_buffer(touch, width, height):
    stride = width * 4
    size = stride * height

    with tempfile.TemporaryFile() as f:
        f.write(b"\x64" * size)
        f.flush()

        fd = f.fileno()
        touch["data"] = mmap.mmap(
            fd, size, mmap.MAP_SHARED, mmap.PROT_READ | mmap.PROT_WRITE
        )
        pool = touch["shm"].create_pool(fd, size)
        touch["buffer"] = pool.create_buffer(
            0, width, height, stride, WlShm.format.argb8888.value
        )
        pool.destroy()


def handle_touch_down(wl_touch, serial, time, surface, id, x, y):
    # touch = wl_touch.user_data
    # touch_paint(touch, x, y, id)
    return 0


def handle_touch_motion(wl_touch, time, id, x, y):
    # touch = wl_touch.user_data
    # touch_paint(touch, x, y, id)
    return 0


def handle_seat_capabilities(wl_seat, capabilities):
    print("capabilities")
    seat = wl_seat.user_data
    touch = seat["touch"]

    if (capabilities & WlSeat.capability.touch.value) and seat["wl_touch"] is None:
        seat["wl_touch"] = wl_seat.get_touch()
        seat["wl_touch"].user_data = touch
        seat["wl_touch"].dispatcher["down"] = handle_touch_down
        # seat['wl_touch'].dispatcher['up'] = handle_touch_up
        seat["wl_touch"].dispatcher["motion"] = handle_touch_motion
    elif not (capabilities & WlSeat.capability.touch.value) and seat["wl_touch"]:
        seat["wl_touch"].destroy()
        seat["wl_touch"] = None
    return 1


def handle_shm_format(wl_shm, fmt):
    print("format")
    touch = wl_shm.user_data

    if fmt == WlShm.format.argb8888.value:
        touch["has_argb"] = True
    return 1


def handle_shell_surface_ping(wl_shell_surface, serial):
    print("ping")
    wl_shell_surface.pong(serial)
    return 1


def handle_registry_global(wl_registry, id_num, iface_name, version):
    print("global", id_num, iface_name)

    touch = wl_registry.user_data
    if iface_name == "wl_compositor":
        touch["compositor"] = wl_registry.bind(id_num, WlCompositor, version)
    elif iface_name == "wl_seat":
        seat = {}
        seat["touch"] = touch
        seat["wl_touch"] = None

        wl_seat = wl_registry.bind(id_num, WlSeat, version)
        wl_seat.dispatcher["capabilities"] = handle_seat_capabilities
        wl_seat.user_data = seat
        seat["seat"] = wl_seat
    elif iface_name == "wl_shell":
        touch["shell"] = wl_registry.bind(id_num, WlShell, version)
    elif iface_name == "wl_shm":
        touch["has_argb"] = False

        shm = wl_registry.bind(id_num, WlShm, version)
        shm.user_data = touch
        shm.dispatcher["format"] = handle_shm_format
        touch["shm"] = shm
    return 1


def touch_create(width, height):
    touch = {}

    # Make the display and get the registry
    touch["display"] = Display()
    touch["display"].connect()

    touch["registry"] = touch["display"].get_registry()
    touch["registry"].user_data = touch
    touch["registry"].dispatcher["global"] = handle_registry_global

    touch["display"].dispatch()
    touch["display"].roundtrip()
    touch["display"].roundtrip()

    if not touch["has_argb"]:
        print("WL_SHM_FORMAT_ARGB32 not available", file=sys.stderr)
        touch["display"].disconnect()
        return

    touch["width"] = width
    touch["height"] = height
    touch["surface"] = touch["compositor"].create_surface()
    touch["shell_surface"] = touch["shell"].get_shell_surface(touch["surface"])
    create_shm_buffer(touch, width, height)

    if touch["shell_surface"]:
        print("shell")
        touch["shell_surface"].dispatcher["ping"] = handle_shell_surface_ping
        touch["shell_surface"].set_toplevel()

    touch["surface"].user_data = touch
    touch["shell_surface"].set_title("simple-touch")

    touch["surface"].attach(touch["buffer"], 0, 0)
    touch["surface"].damage(0, 0, width, height)
    touch["surface"].commit()

    return touch


def main():
    touch = touch_create(600, 500)

    while touch["display"].dispatch() != -1:
        pass

    touch["display"].disconnect()


if __name__ == "__main__":
    main()



import os
import tarfile
import urllib.request

wayland_version = '1.18.0'
protocols_version = '1.20'

wayland_source = 'https://cgit.freedesktop.org/wayland/wayland/plain/protocol/wayland.xml?id={}'.format(wayland_version)
protocols_source = 'https://wayland.freedesktop.org/releases/wayland-protocols-{}.tar.xz'.format(protocols_version)


def protocols_build(output_dir):
    from pywayland.scanner import Protocol

    # first, we download the wayland.xml file
    wayland_file = 'wayland.xml'
    urllib.request.urlretrieve(wayland_source, wayland_file)

    # download the protocols file and extract it
    protocol_dest = 'wayland-protocols-{}'.format(protocols_version)
    urllib.request.urlretrieve(protocols_source, protocol_dest + '.tar.xz')
    with tarfile.open(protocol_dest + '.tar.xz') as f:
        f.extractall()

    # walk the directory and generate all the protocols
    protocol_files = [wayland_file] + sorted([
        os.path.join(dirpath, filename)
        for dirpath, _, filenames in os.walk(protocol_dest)
        for filename in filenames
        if os.path.splitext(filename)[1] == ".xml"
    ], reverse=True)

    protocols = [Protocol.parse_file(protocol_file) for protocol_file in protocol_files]
    protocol_imports = {
        interface.name: protocol.name
        for protocol in protocols
        for interface in protocol.interface
    }
    for protocol in protocols:
        protocol.output(output_dir, protocol_imports)



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
from functools import wraps

from . import lib


def ensure_valid(func):
    @wraps(func)
    def wrapper(self, *args, **kwargs):
        if self._ptr is None:
            raise ValueError(
                "{cls} object has been destroyed".format(cls=self.__class__.__name__)
            )
        return func(self, *args, **kwargs)

    return wrapper


class AnonymousFile:
    """Anonymous file object

    Provides access to anonymous file objects that can be used by Wayland
    clients to render to surfaces.  Uses a method similar to Weston to open an
    anonymous file, so XDG_RUNTIME_DIR must be set for this to work properly.

    This class provides a content manager, that is, it can be used with Python
    ``with`` statements, where the value returned is the file descriptor.
    """

    def __init__(self, size):
        self.size = size
        self.fd = None

    def __enter__(self):
        self.open()

        return self.fd

    def __exit__(self, exc_type, exc_value, traceback):
        self.close()

    def open(self):
        """Open an anonymous file

        Opens the anonymous file and sets the ``fd`` property to the file
        descriptor that has been opened.
        """
        if self.fd is not None:
            raise IOError("File is already open")
        self.fd = lib.os_create_anonymous_file(self.size)
        if self.fd < 0:
            raise IOError("Unable to create anonymous file")

    def close(self):
        """Close the anonymous file

        Closes the file descriptor and sets the ``fd`` property to ``None``.
        Does nothing if the file is not open.
        """
        if self.fd is None:
            return

        os.close(self.fd)
        self.fd = None



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import importlib
import os
import shutil
import sys

import sphinx_rtd_theme


# -- Mock necessary classes -----------------------------------------------

from unittest.mock import MagicMock

sys.path.insert(0, os.path.abspath('.'))  # noqa: F402
sys.path.insert(0, os.path.abspath('..'))  # noqa: F402

MOCK_MODULES = ['pywayland._ffi']  # noqa: F402
sys.modules.update((mod_name, MagicMock()) for mod_name in MOCK_MODULES)  # noqa: F402

from pywayland import __version__

# -- Build pywayland.protocol w/docs --------------------------------------

from protocol_build import wayland_version, protocols_version, protocols_build

protocol_build_dir = '../pywayland/protocol/'
protocol_doc_dir = 'module/protocol'

index_header = """\
.. _protocol:

Protocol Modules
================

Wayland protocols built against Wayland {} and Wayland Protocols {}.

.. toctree::
   :maxdepth: 2

""".format(wayland_version, protocols_version)

protocol_header = """\
.. module:: pywayland.protocol.{module}

{module} Module
{empty:=^{len}}======="""

protocol_rst = """\
{protocol}
{empty:-^{len}}

.. wl_protocol:: pywayland.protocol.{module} {protocol}"""


# There is probably a better way to do this in Sphinx, templating or something
# ... but this works
def protocol_doc(input_dir, output_dir):
    modules = os.listdir(input_dir)
    modules = [
        module for module in modules
        if os.path.isdir(os.path.join(input_dir, module)) and module != '__pycache__'
    ]

    existing_files = [
        filename for filename in os.listdir(output_dir) if filename != "index.rst"
    ]
    rm_files = [
        filename for filename in existing_files if os.path.splitext(filename)[0] not in modules
    ]
    for rm_file in rm_files:
        if os.path.isdir(rm_file):
            shutil.rmtree(os.path.join(output_dir, rm_file))
        else:
            os.remove(os.path.join(output_dir, rm_file))

    # Write out the index file
    index_file = os.path.join(output_dir, 'index.rst')
    if os.path.exists(index_file):
        with open(index_file) as f:
            existing_index = f.read()
    else:
        existing_index = ""

    generated_index = index_header + "".join('   {}\n'.format(m) for m in sorted(modules))

    if existing_index != generated_index:
        with open(index_file, 'w') as f:
            f.write(generated_index)

    for module in modules:
        output = [
            protocol_header.format(
                module=module,
                len=len(module),
                empty=''
            )
        ]

        # get all the python files that we want to document
        doc_files = os.listdir(os.path.join(input_dir, module))
        doc_files = [
            os.path.splitext(doc_file)[0] for doc_file in doc_files
            if doc_file != '__init__.py' and os.path.splitext(doc_file)[1] == '.py'
        ]

        # build the rst for each protocol
        for doc_file in doc_files:
            mod = importlib.import_module('pywayland.protocol.{}.{}'.format(module, doc_file))
            # Get out the name of the class in the module
            class_name = ''.join(x.capitalize() for x in doc_file.split('_'))
            for mod_upper in dir(mod):
                if mod_upper == class_name:
                    break
            else:
                raise RuntimeError("Unable to find module: {}, {}".format(doc_file, mod))

            output.append(protocol_rst.format(
                module=module,
                protocol=mod_upper,
                len=len(mod_upper),
                empty=''
            ))

        # build the index.rst for the module
        module_file = os.path.join(output_dir, '{}.rst'.format(module))
        protocol_output = "\n\n".join(output)

        # if file exists and is unchanged, skip
        if os.path.exists(module_file):
            with open(module_file) as f:
                existing_output = f.read()

            if existing_output == protocol_output:
                continue

        with open(module_file, 'w') as f:
            f.write("\n\n".join(output))


# Build the protocol directory on RTD
if os.environ.get('READTHEDOCS', None):
    protocols_build(protocol_build_dir)

# Re-build the protocol documentation directory
if not os.path.exists(protocol_doc_dir):
    os.makedirs(protocol_doc_dir)

protocol_doc(protocol_build_dir, protocol_doc_dir)

# -- General configuration ------------------------------------------------
extensions = [
    'sphinx.ext.autodoc',
    'sphinx_wl_protocol'
]

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix of source filenames.
source_suffix = '.rst'

# The master toctree document.
master_doc = 'index'

# General information about the project.
project = 'pywayland'
copyright = '2016, Sean Vig'

# The short X.Y version.
version = __version__.split("a")[0]
# The full version, including alpha/beta/rc tags.
release = __version__

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
exclude_patterns = ['_build']

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'


# -- Options for HTML output ----------------------------------------------

# Set the html_theme when building locally
html_theme = 'sphinx_rtd_theme'
html_theme_path = [sphinx_rtd_theme.get_html_theme_path()]

# Output file base name for HTML help builder.
htmlhelp_basename = 'pywaylanddoc'


# -- Options for LaTeX output ---------------------------------------------

latex_elements = {}

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title,
#  author, documentclass [howto, manual, or own class]).
latex_documents = [
    ('index', 'pywayland.tex', 'pywayland Documentation',
     'Sean Vig', 'manual'),
]


# -- Options for manual page output ---------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    ('index', 'pywayland', 'pywayland Documentation',
     ['Sean Vig'], 1)
]


# -- Options for Texinfo output -------------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
    ('index', 'pywayland', 'pywayland Documentation',
     'Sean Vig', 'pywayland', 'Python bindings for the libwayland library',
     'Miscellaneous'),
]



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import, print_function

import mmap
import os
import sys

this_file = os.path.abspath(__file__)
this_dir = os.path.split(this_file)[0]
root_dir = os.path.split(this_dir)[0]
pywayland_dir = os.path.join(root_dir, "pywayland")
if os.path.exists(pywayland_dir):
    sys.path.append(root_dir)

from pywayland.client import Display  # noqa: E402
from pywayland.protocol.wayland import WlCompositor, WlShell, WlShm  # noqa: E402
from pywayland.utils import AnonymousFile  # noqa: E402

WIDTH = 480
HEIGHT = 256

MARGIN = 10


class Window(object):
    def __init__(self):
        self.buffer = None
        self.compositor = None
        self.shell = None
        self.shm = None
        self.shm_data = None
        self.surface = None

        self.line_pos = MARGIN
        self.line_speed = +1


def shell_surface_ping_handler(shell_surface, serial):
    shell_surface.pong(serial)
    print("pinged/ponged")


def shm_format_handler(shm, format_):
    if format_ == WlShm.format.argb8888.value:
        s = "ARGB8888"
    elif format_ == WlShm.format.xrgb8888.value:
        s = "XRGB8888"
    elif format_ == WlShm.format.rgb565.value:
        s = "RGB565"
    else:
        s = "other format"
    print("Possible shmem format: {}".format(s))


def registry_global_handler(registry, id_, interface, version):
    window = registry.user_data
    if interface == "wl_compositor":
        print("got compositor")
        window.compositor = registry.bind(id_, WlCompositor, version)
    elif interface == "wl_shell":
        print("got shell")
        window.shell = registry.bind(id_, WlShell, version)
    elif interface == "wl_shm":
        print("got shm")
        window.shm = registry.bind(id_, WlShm, version)
        window.shm.dispatcher["format"] = shm_format_handler


def registry_global_remover(registry, id_):
    print("got a registry losing event for {}".format(id))


def create_buffer(window):
    stride = WIDTH * 4
    size = stride * HEIGHT

    with AnonymousFile(size) as fd:
        window.shm_data = mmap.mmap(
            fd, size, prot=mmap.PROT_READ | mmap.PROT_WRITE, flags=mmap.MAP_SHARED
        )
        pool = window.shm.create_pool(fd, size)
        buff = pool.create_buffer(0, WIDTH, HEIGHT, stride, WlShm.format.argb8888.value)
        pool.destroy()
    return buff


def create_window(window):
    window.buffer = create_buffer(window)
    window.surface.attach(window.buffer, 0, 0)
    window.surface.commit()


def redraw(callback, time, destroy_callback=True):
    window = callback.user_data
    if destroy_callback:
        callback._destroy()

    paint(window)
    window.surface.damage(0, 0, WIDTH, HEIGHT)

    callback = window.surface.frame()
    callback.dispatcher["done"] = redraw
    callback.user_data = window

    window.surface.attach(window.buffer, 0, 0)
    window.surface.commit()


def paint(window):
    mm = window.shm_data
    # clear
    mm.seek(0)
    mm.write(b"\xff" * 4 * WIDTH * HEIGHT)

    # draw progressing line
    mm.seek((window.line_pos * WIDTH + MARGIN) * 4)
    mm.write(b"\x00\x00\x00\xff" * (WIDTH - 2 * MARGIN))
    window.line_pos += window.line_speed

    # maybe reverse direction of progression
    if window.line_pos >= HEIGHT - MARGIN or window.line_pos <= MARGIN:
        window.line_speed = -window.line_speed


def main():
    window = Window()

    display = Display()
    display.connect()
    print("connected to display")

    registry = display.get_registry()
    registry.dispatcher["global"] = registry_global_handler
    registry.dispatcher["global_remove"] = registry_global_remover
    registry.user_data = window

    display.dispatch(block=True)
    display.roundtrip()

    if window.compositor is None:
        raise RuntimeError("no compositor found")
    elif window.shell is None:
        raise RuntimeError("no shell found")
    elif window.shm is None:
        raise RuntimeError("no shm found")

    window.surface = window.compositor.create_surface()

    shell_surface = window.shell.get_shell_surface(window.surface)
    shell_surface.set_toplevel()
    shell_surface.dispatcher["ping"] = shell_surface_ping_handler

    frame_callback = window.surface.frame()
    frame_callback.dispatcher["done"] = redraw
    frame_callback.user_data = window

    create_window(window)
    redraw(frame_callback, 0, destroy_callback=False)

    while display.dispatch(block=True) != -1:
        pass

    import time

    time.sleep(1)

    display.disconnect()


if __name__ == "__main__":
    main()



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import importlib
import inspect

from docutils.statemachine import ViewList
from jinja2 import Template
from docutils import nodes
from docutils.parsers.rst import Directive
from sphinx.util.docstrings import prepare_docstring
from sphinx.util.inspect import getargspec
from sphinx.util.nodes import nested_parse_with_titles


# Adapted from sphinxcontrib-httpdomain
def import_object(module_name, class_name):
    mod = importlib.import_module(module_name)
    return getattr(mod, class_name)


def format_args(func):
    arg_spec = getargspec(func)
    if arg_spec[0][0] == 'self':  # Should always be true
        del(arg_spec[0][0])
    return formatargspec(*arg_spec)


def formatargspec(*argspec):
    # most recent version of sphinx.ext.autodoc uses:
    # from sphinx.util.inspect import object_description
    # return inspect.formatargspec(*argspec, formatvalue=lambda x: '=' + object_description(x))
    # RTD uses an old sphinx version without object_description, but this doesn't seem to make any difference for this
    return inspect.formatargspec(*argspec)


wl_protocol_template = Template("""
.. autoclass:: {{ module }}.{{ class_name }}
    {% for func, opcode, sig, docs in requests %}
    .. method:: {{ func }} {{ sig }}

        .. rubric:: Request -- opcode {{ opcode}} (attached to :class:`~pywayland.server.resource.Resource` instance)

{% for doc in docs %}
        {{ doc }}{% endfor %}{% endfor %}
    {% for func, opcode, sig, docs in events %}
    .. method:: {{ func }} {{ sig }}

        .. rubric:: Event -- opcode {{ opcode}} (attached to :class:`~pywayland.client.proxy.Proxy` instance)

{% for doc in docs %}
        {{ doc }}{% endfor %}{% endfor %}
""")


class WlProtocol(Directive):
    required_arguments = 2

    def make_rst(self):
        module_name, class_name = self.arguments[:2]
        obj = import_object(module_name, class_name)
        events = [
            (event.py_func.__name__, opcode, format_args(event.py_func), prepare_docstring(event.py_func.__doc__))
            for opcode, event in enumerate(obj.events)
        ]
        reqs = [(req.py_func.__name__, opcode, format_args(req.py_func),
                 prepare_docstring(req.py_func.__doc__)) for opcode, req in enumerate(obj.requests)]
        context = {
            'module': module_name,
            'class_name': class_name,
            'obj': obj,
            'events': events,
            'requests': reqs
        }
        rst = wl_protocol_template.render(**context)
        for line in rst.splitlines():
            yield line

    def run(self):
        node = nodes.section()
        node.document = self.state.document
        result = ViewList()
        for line in self.make_rst():
            result.append(line, '<{0}>'.format(self.__class__.__name__))
        nested_parse_with_titles(self.state, result, node)
        return node.children


def setup(app):
    app.add_directive('wl_protocol', WlProtocol)



from typing import overload, Any, Callable, TypeVar, Union

class Cdata: ...

class CdataArray:
    def __setitem__(self, idx: int, elem: Cdata) -> None: ...

# built-in cdata types
class NullCdata(Cdata): ...
class CharCdata(Cdata): ...

# custom cdata types
class ClientCdata(Cdata): ...
class DisplayCdata(Cdata): ...
class EventLoopCdata(Cdata): ...
class ListenerCdata(Cdata): ...
class QueueCdata(Cdata): ...
class ResourceCdata(Cdata): ...

_FuncType = Callable[..., Any]
_F = TypeVar("_F", bound=_FuncType)
_CdataT = TypeVar("_CdataT", bound=Cdata)

class ffi:
    NULL: NullCdata
    @overload
    @staticmethod
    def new(cdecl: str) -> Cdata: ...
    @overload
    @staticmethod
    def new(cdecl: str, init: Any) -> CdataArray: ...
    @staticmethod
    def gc(
        cdata: _CdataT, destructor: Callable[[_CdataT], None], size: int = 0
    ) -> _CdataT: ...
    @staticmethod
    def string(cdata: CharCdata) -> str: ...
    @staticmethod
    def release(cdata: Cdata) -> None: ...
    @staticmethod
    def def_extern() -> Callable[[_F], _F]: ...
    @staticmethod
    def from_handle(cdata: Cdata) -> Any: ...

class lib:
    WAYLAND_VERSION_MAJOR: int
    WAYLAND_VERSION_MINOR: int
    WAYLAND_VERSION_MICRO: int
    # Event loop functionality
    @staticmethod
    def wl_event_loop_create() -> EventLoopCdata: ...
    @staticmethod
    def wl_event_loop_destroy(event_loop: EventLoopCdata) -> None: ...
    # Display functionality
    @staticmethod
    def wl_display_create() -> DisplayCdata: ...
    @staticmethod
    def wl_display_destroy(display: DisplayCdata) -> None: ...
    @staticmethod
    def wl_display_destroy_clients(display: DisplayCdata) -> None: ...
    @staticmethod
    def wl_display_flush_clients(display: DisplayCdata) -> None: ...
    @staticmethod
    def wl_display_get_event_loop(display: DisplayCdata) -> EventLoopCdata: ...
    @staticmethod
    def wl_display_add_socket(display: DisplayCdata, name: bytes) -> int: ...
    @staticmethod
    def wl_display_add_socket_auto(display: DisplayCdata) -> CharCdata: ...
    @staticmethod
    def wl_display_terminate(display: DisplayCdata) -> None: ...
    @staticmethod
    def wl_display_run(display: DisplayCdata) -> None: ...
    @staticmethod
    def wl_display_get_serial(display: DisplayCdata) -> int: ...
    @staticmethod
    def wl_display_next_serial(display: DisplayCdata) -> int: ...
    @staticmethod
    def wl_display_init_shm(display: DisplayCdata) -> int: ...
    @staticmethod
    def wl_display_add_shm_format(display: DisplayCdata, format: int) -> int: ...
    @staticmethod
    def wl_event_queue_destroy(queue: QueueCdata) -> None: ...
    @staticmethod
    def wl_display_connect(name: Union[bytes, NullCdata]) -> DisplayCdata: ...
    @staticmethod
    def wl_display_connect_to_fd(fd: int) -> DisplayCdata: ...
    @staticmethod
    def wl_display_disconnect(display: DisplayCdata) -> None: ...
    @staticmethod
    def wl_display_get_fd(display: DisplayCdata) -> int: ...
    @staticmethod
    def wl_display_dispatch(display: DisplayCdata) -> int: ...
    @staticmethod
    def wl_display_dispatch_pending(display: DisplayCdata) -> int: ...
    @staticmethod
    def wl_display_dispatch_queue(display: DisplayCdata, queue: QueueCdata) -> int: ...
    @staticmethod
    def wl_display_dispatch_queue_pending(
        display: DisplayCdata, queue: QueueCdata
    ) -> int: ...
    @staticmethod
    def wl_display_roundtrip(display: DisplayCdata) -> int: ...
    @staticmethod
    def wl_display_roundtrip_queue(display: DisplayCdata, queue: QueueCdata) -> int: ...
    @staticmethod
    def wl_display_get_error(display: DisplayCdata) -> int: ...
    @staticmethod
    def wl_display_read_events(display: DisplayCdata) -> int: ...
    @staticmethod
    def wl_display_prepare_read(display: DisplayCdata) -> int: ...
    @staticmethod
    def wl_display_prepare_read_queue(
        display: DisplayCdata, queue: QueueCdata
    ) -> int: ...
    @staticmethod
    def wl_display_flush(display: DisplayCdata) -> int: ...
    @staticmethod
    def wl_display_create_queue(display: DisplayCdata) -> QueueCdata: ...
    @staticmethod
    def wl_client_create(display: DisplayCdata, fd: int) -> ClientCdata: ...
    @staticmethod
    def wl_client_destroy(client: ClientCdata) -> None: ...
    @staticmethod
    def wl_client_flush(client: ClientCdata) -> None: ...
    @staticmethod
    def wl_client_add_destroy_listener(
        client: ClientCdata, listener: ListenerCdata
    ) -> None: ...
    @staticmethod
    def wl_client_get_object(client: ClientCdata, id: int) -> ResourceCdata: ...
    @staticmethod
    def wl_resource_get_user_data(resource: ResourceCdata) -> Cdata: ...
    WL_EVENT_READABLE: int
    WL_EVENT_WRITABLE: int
    WL_EVENT_HANGUP: int
    WL_EVENT_ERROR: int
